#!/usr/bin/python
from datetime import datetime, date, timedelta
import time
import random
import os
import gconf
import xml.etree.ElementTree as ET

backgrounds_filename = os.path.join(os.environ['HOME'], '.gnome2', 'backgrounds.xml')
config_filename = os.path.join(os.environ['HOME'], '.change-background.xml')

def load_config():
    config = {
        'min_period': timedelta(3),
        'day_chance': 0.1,
        'min_chance': 0.3,
        'last_run': datetime.min,
        'dates': {},
    }

    def _set_setting(name, constructor):
        el = xml.find('settings/%s' % name)
        if el is not None:
            config[name] = constructor(el.text)

    def _date(value):
        return datetime.strptime(value, '%Y-%m-%d').date()

    def _datetime(value):
        return datetime.strptime(value, '%Y-%m-%d %H:%M:%S')

    if os.path.exists(config_filename):
        xml = ET.parse(config_filename)
        _set_setting('min_period', lambda v: timedelta(int(v)))
        _set_setting('day_chance', float)
        _set_setting('min_chance', float)
        _set_setting('last_run', _datetime)
        dates = xml.findall('dates/wallpaper')
        config['dates'] = dict((el.attrib['filename'], _date(el.attrib['last-shown'])) for el in dates)

    return config

def save_config(config, backgrounds):
    root = ET.Element('change-background-config')

    def _store_setting(name, serializer=str):
        ET.SubElement(settings, name).text = serializer(config[name])

    settings = ET.SubElement(root, 'settings')
    _store_setting('min_period', lambda v: str(v.days))
    _store_setting('day_chance')
    _store_setting('min_chance')
    _store_setting('last_run', lambda v: v.strftime('%Y-%m-%d %H:%M:%S'))

    dates = ET.SubElement(root, 'dates')
    for b in backgrounds:
        ET.SubElement(dates, 'wallpaper', {'filename': b['filename'], 'last-shown': str(b['date'])})

    xml = ET.ElementTree(root)
    xml.write(config_filename, encoding='utf-8')

def load_backgrounds(config):

    def bg_dict(el):
        return dict((c.tag, c.text) for c in el)

    elements = ET.parse(backgrounds_filename).findall('wallpaper')
    bg_dicts = [bg_dict(el) for el in elements if el.attrib['deleted'] == 'false']
    for d in bg_dicts:
        d['date'] = config['dates'].get(d['filename'], date.min)
    return sorted(bg_dicts, key=lambda d: d['date'])

def set_background(background):
    client = gconf.client_get_default()
    key_map = {
        'filename': 'picture_filename',
        'shade_type': 'color_shading_type',
        'options': 'picture_options',
        'pcolor': 'primary_color',
        'scolor': 'secondary_color',
    }
    for key, value in background.items():
        if key in key_map:
            client.set_string('/desktop/gnome/background/%s' % key_map[key], value)

def enough(config, last_shown):
    period = date.today() - last_shown
    return period > config['min_period'] and \
           (random.random() < (period - config['min_period']).days * config['day_chance'] + config['min_chance'])

def main():
    config = load_config()
    if datetime.now() - config['last_run'] < timedelta(1):
        return
    backgrounds = load_backgrounds(config)
    if not backgrounds:
        return
    if enough(config, backgrounds[-1]['date']):
        background = random.choice(backgrounds[:len(backgrounds) / 2])
        set_background(background)
        background['date'] = date.today()
    config['last_run'] = datetime.now()
    save_config(config, backgrounds)

if __name__ == '__main__':
    while 1:
        main()
        time.sleep(random.randint(3600, 7200))
